﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using Telerik.WinControls.UI;

namespace CustomTypeConverterDemo
{
    public class CustomTypeConverter : TypeConverter
    {
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            if (sourceType == typeof(string))
            {
                return true;
            }

            return base.CanConvertFrom(context, sourceType);
        }

        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            if (destinationType == typeof(string))
            {
                return true;
            }

            return base.CanConvertTo(context, destinationType);
        }

        public override bool GetPropertiesSupported(ITypeDescriptorContext context)
        {
            return true;
        }

        public override bool GetCreateInstanceSupported(ITypeDescriptorContext context)
        {
            return false;
        }

        public override PropertyDescriptorCollection GetProperties(ITypeDescriptorContext context, object value, Attribute[] attributes)
        {
            List<PropertyDescriptor> result = new List<PropertyDescriptor>();

            PropertyDescriptorCollection properties = TypeDescriptor.GetProperties(((PropertyGridItem)context).Value, attributes);

            result.Add(properties["WorktypeNo"]);
            result.Add(properties["WorktypeName"]);

            return new PropertyDescriptorCollection(result.ToArray());
        }

        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
        {
            if (destinationType == typeof(string))
            {
                return ((WorktypeInfo)value).WorktypeNo.ToString() + ", " + ((WorktypeInfo)value).WorktypeName;
            }

            return base.ConvertTo(context, culture, value, destinationType);
        }

        public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
        {
            string[] strings = value.ToString().Split(',');
            if (strings.Length != 2)
            {
                throw new InvalidExpressionException("The input string was not in the correct format.");
            }

            return new WorktypeInfo(strings[0], strings[1]);
        }
    }
}
